window.ShowLinksSetup = function () {

    var customURL = app.getValue('showLinks_CustomSearchEngine', '');
    var enableCustom = app.getValue('showLinks_EnableCustomSearchEngine', false);

    window.TrackShortcuts = [
        {
            id: 1,
            grouporder: 10,
            order: 10,
            icon: 'google',
            title: 'Google',
            execute: function (type, prop, item) {
                uitools.openWebSearch('http://www.google.com/search?q=%s', type, item);
            }
        },
        {
            id: 2,
            grouporder: 10,
            order: 20,
            icon: 'wikipedia',
            title: 'Wikipedia',
            execute: function (type, prop, item) {
                uitools.openWebSearch('http://www.google.com/search?btnI=I\'m Feeling Lucky&q=%s site:wikipedia.org', type, item);
            }
        },
        {
            id: 3,
            grouporder: 10,
            order: 30,
            icon: 'youtube',
            title: 'YouTube',
            execute: function (type, prop, item) {
                uitools.openWebSearch('http://www.youtube.com/results?search_query=%s', type, item);
            }
        },
        {
            id: 4,
            grouporder: 10,
            order: 40,
            icon: 'spotify',
            title: 'Spotify',
            execute: function (type, prop, item) {
                uitools.openWebSearch('https://open.spotify.com/search/%s', type, item);
            }
        },
        {
            id: 5,
            grouporder: 10,
            order: 50,
            icon: 'discogs',
            title: 'Discogs',
            execute: function (type, prop, item) {
                uitools.openWebSearch('https://www.discogs.com/search/?q=%s&type=all', type, item);
            }
        },
        {
            id: 6,
            grouporder: 10,
            order: 60,
            icon: 'allmusic',
            title: 'AllMusic',
            execute: function (type, prop, item) {
                uitools.openWebSearch('https://www.allmusic.com/search/all/%s', type, item);
            }
        },
        {
            id: 99,
            grouporder: 20,
            order: 99,
            icon: 'library',
            title: _('Library'),
            execute: function (type, prop, item) {
                if (navigationHandlers[prop]) {
                    //#17604: Navigating to album from just album name did not navigate correctly
                    navigationHandlers[prop].navigate(item);
                }
            }
        }
    ];

    //Add custom url if it's enabled
    if (enableCustom)
        window.TrackShortcuts.push({
            id: 50,
            grouporder: 10,
            order: 99,
            title: _('Custom'),
            execute: function (type, prop, item) {
                uitools.openWebSearch(customURL, type, item);
            }
        })

    var handleShortcut = function (shortcut, type, prop, item) {
        var shortcutExecuted = false;
        for (var i = 0; i < TrackShortcuts.length; i++) {
            if (TrackShortcuts[i].id === shortcut) {
                if (TrackShortcuts[i].execute) {
                    TrackShortcuts[i].execute(type, prop, item);
                    shortcutExecuted = true;
                }
                break;
            }
        }
        if (!shortcutExecuted) {
            TrackShortcuts[0].execute(type, prop, item);
        }
    };

    window.uitools.tracklistFieldDefs.title.shortcutFunc = function (item, shortcutID) {
        handleShortcut(shortcutID, 'title', 'title', item);
    }

    window.uitools.tracklistFieldDefs.artist.shortcutFunc = function (item, shortcutID) {
        handleShortcut(shortcutID, 'artist', 'artist', item);
    }

    window.uitools.tracklistFieldDefs.album.shortcutFunc = function (item, shortcutID) {
        handleShortcut(shortcutID, 'album', 'album', item);
    }

    window.uitools.tracklistFieldDefs.albumArtist.shortcutFunc = function (item, shortcutID) {
        handleShortcut(shortcutID, 'albumArtist', 'albumArtist', item);
    }

    window.uitools.tracklistFieldDefs.series.shortcutFunc = function (item, shortcutID) {
        handleShortcut(shortcutID, 'series', 'album', item);
    }
    
    TrackListView.prototype.override({
        initialize: function ($super, elem, params) {
            $super(elem, params);
            this._showShortcuts = true;
    
            var loadSettings = () => {
                var sett = settings.get('Options');
                this._activeShortcut = sett.Options.ActiveTrackShortcutType;
            };
            loadSettings();

            this.localListen(settings.observer, 'change', () => {
                loadSettings();
            });
        },
    });

    TrackListView.prototype.getShortcutContextMenu = (parent, column) => {
        var presetColumns = [];
        TrackShortcuts.sort(function (i1, i2) {
            return i1.order - i2.order;
        });
        forEach(TrackShortcuts, function (shortcut, idx) {
            presetColumns.push({
                action: {
                    title: shortcut.title,
                    icon: shortcut.icon,
                    execute: function () {
                        if (parent.parentElement.itemIndex >= 0) {
                            var LV = parent.parentElement.parentListView;
                            var item = undefined, songData = undefined;
                            item = LV.focusedItem;
                            // if getFastSD is a function, that means this is a playlist entry, not a song data object
                            if (typeof item.getFastSD === 'function') {
                                songData = item.getFastSD(songData);
                            }
                            else {
                                songData = item;
                            }
                            column.shortcutFunc(songData, shortcut.id);
                        }
                    }
                },
                grouporder: shortcut.grouporder,
            });
        });
        return new Menu(presetColumns, {
            parent: parent
        });
    }

}

if (window.isMainWindow)
    window.ShowLinksSetup();
